﻿// EntryList object.
// Manages the UI:
//  - The frame
//  - The content
//  - Adding entry items to the list
//  - Scrolling
var entryList; 
// Entries received from the Salesforce.
var lastEntries;
// The current entry displayed in the details view.
var currentEntry = null;
/*
* hash map of all object codes
*/
var sObjectHashMap = new Array();
/*
* Detail Content 
*/
var detailHtmlContent;
/*
Timer ID
*/
var iTimerID;
/*
* Start gadget handlers
*/
plugin.onShowOptionsDlg = showOptionsDialogHandler; 
/* Options Initialization Dialog */
function showOptionsDialogHandler(wnd) {
  /* General Title */
	wnd.AddControl(gddWndCtrlClassLabel, 0, "", "Please specify user name and password to " + 
								"access to Salesforce. We won't share this data with anyone else.", 10, 10, 280,30);
  /* User name */
	wnd.AddControl(gddWndCtrlClassLabel, 0, "", "User name:", 10, 40, 90,30);
  var controlIDForUserName = wnd.AddControl(gddWndCtrlClassEdit, 0,"userName", "", 90, 40, 190, 20);
	/* User Password */
	wnd.AddControl(gddWndCtrlClassLabel, 0, "", "User password:", 10, 70, 90,30);
  var controlIDForUserPassword = wnd.AddControl(gddWndCtrlClassEdit, gddWndCtrlTypeEditPassword,"userPassword", "", 90, 70, 190, 20);
	/* Init Options Dialog */
  if (options.exists("userName")) {
		controlIDForUserName.value = options.getValue("userName");
  }
  if (options.exists("userPassword")) {
			controlIDForUserPassword.value = options.getValue("userPassword");
	}
  /* set event onClose */
  wnd.onClose = onOptionsDialogClosed;
}
/* OnSave handler for Options dialog */
function onOptionsDialogClosed(wnd, code) {
	if (code==gddIdOK) {
		// set values
		var userName = wnd.GetControl("userName");
		var userPassword = wnd.GetControl("userPassword");
		// make logon to Salesforce
		if (!tryLogon(userName.value, userPassword.value, false)) {
			return false;
		} else {
			// setup them as options
			options.putValue("userName", userName.value);
			options.putValue("userPassword", userPassword.value);
			// fetching data
			getSObjectHashMap(sforce.connection);
		}
  }
}
/*
* Try logon to Salesforce.com via API
*/
function tryLogon(userName, password, relogin) {
		// do not do logon if user didn't change his password
		if ((userName==options.getValue("userName")) && relogin==false) {
			return true;
		}
		try {
			sforce.connection.serverUrl = "https://www.salesforce.com/services/Soap/u/11.0";
			var loginResult = sforce.connection.login(userName, password);
			options.putValue("sessionId", loginResult.sessionId); // store to options
			options.putValue("serverUrl", loginResult.serverUrl); // store to options
			options.putValue("actorId", loginResult.userId); // store to options
			// setup to sforce serverUrl
			sforce.connection.serverUrl = loginResult.serverUrl;
			return true;
		} catch (e) {
			view.alert(e.faultstring); // to get error code please use faultcode
			return false;
		}
}

function onSize() {
  entryList.resize(view.width, view.height);
} 

function onOpen() {
	// read detail html content
	detailHtmlContent = gadget.storage.openText("details.html");
  // Create the EntryList.
  entryList = new EntryList(container, CONFIG_THEME_DIR);
  // Call draw with no data to display default error message.
  draw(null); 
	if (!options.exists("userName")) {
		// show text
		entryList.displayMessage(strings.RUN_OPTIONS);		
	} else {		
		iTimerID = view.setInterval("updateGadgetState()", 1000*60*10); // each 10 minutes
		updateGadgetState();
	}
}
/*
* Draw gadget layout
*/
function draw(entries) {
  entryList.clearItems();

  //debug.trace('Adding entries.');
  // If no entry data, indicate error.
  if (!entries) {
    //debug.trace('No entries data, displaying default error.');
    entryList.displayMessage(strings.TRYING_TO_CONNECT);
  } else {
    for (var i = 0; i < entries.length && i < CONFIG_MAX_ENTRIES; ++i) {
      entryList.addItem(entries[i]);
    }
    lastEntries = entries;
  }

  entryList.resize(view.width, view.height);
} 
// updates gadget state
function updateGadgetState() {
	if (options.exists("userName")) {
		// setup sessionId
		sforce.connection.sessionId = options.getValue("sessionId");
		sforce.connection.serverUrl = options.getValue("serverUrl");
		// do updating
		getSObjectHashMap(sforce.connection);
	} else {
		// show text and destroy timer
		entryList.displayMessage(strings.RUN_OPTIONS);
		view.clearInterval(iTimerID);
	}
}

function failureResult(e) {
	alert(e);
}

function failureResultGlobal(e) {
  if (e.faultcode=="sf:INVALID_SESSION_ID") {
		// show message that we are doing relogin
		entryList.displayMessage(strings.RELOGIN_MSG);
		// relogon
		tryLogon(options.getValue("userName"), options.getValue("userPassword"), true);
		// repeat operation once again
		getSObjectHashMap(sforce.connection);
	}
}

/* 
* Retrieve information about all SFDC objects
*/
function getSObjectHashMap(sforcePort) {
	var callback = {
      // onSuccess callback function
			onSuccess: fetchGlobalDescription,
      // onFailure callback function
      onFailure: failureResultGlobal
  };
	try {
		sforcePort.describeSObject("ProcessInstance", callback);
		// clear EntryList
		entryList.clearItems();
		// show text
		entryList.displayMessage(strings.FETCH_GLOBAL_DESCRIPTION);
	} catch (e) {
		view.alert(e);
	}
}

function fetchGlobalDescription(sObject) {
  // get from sObject list of possible sobjects
  if (sObject!=null) {
		var fields = sObject.getArray("fields");
		if (fields!=null && fields.length>0) {
			var objects = [];
			for (var i=0; i<fields.length; i++) {
				if (fields[i]!=null && fields[i].name!=null && fields[i].name.toLowerCase()=="targetobjectid" &&
							fields[i].referenceTo!=null) 
				objects = fields[i].referenceTo;
			}
			var callback = {
					//onSuccess
					onSuccess: constructSObjectDescriptor,
					// onFailure call back function
					onFailure: failureResult
			};
			try {
			// after that let's ask Salesforce about keyPrefix for all SObjects
			debug.trace("Get description for objects:"+objects);
			sforce.connection.describeSObjects(objects, callback);
			} catch (e) {
				view.alert(e.faultstring);
			}
		} else {
			// show text
			entryList.displayMessage(strings.INVALID_PROCESS_INSTANCE);
		}
	} else {
		// show text
		entryList.displayMessage(strings.INVALID_PROCESS_INSTANCE);
  }
}

function constructSObjectDescriptor(arrayOfSObjects) {
	if (arrayOfSObjects!=null) {
		for (var i=0; i<arrayOfSObjects.length; i++) {
			var sobject = {};
			// setup name
			debug.trace("SObject name:"+arrayOfSObjects[i].name);
			debug.trace("SObject keyPrefix:"+arrayOfSObjects[i].keyPrefix);
			sobject['name'] = arrayOfSObjects[i].name;
			// url detail page
			sobject['urldetail'] = arrayOfSObjects[i].urlDetail;
			// url edit page
			sobject['urledit'] = arrayOfSObjects[i].urlEdit;
			// save it
			sObjectHashMap[arrayOfSObjects[i].keyPrefix] = sobject;
		}
	}
	try {
		fetchRequests(); 
	} catch (e) {
		view.alert(e);
	}
}

function fetchRequests() {
  // let's try here to query all pending approval requests
  var callback = {
      // onSuccess callback function
			onSuccess: pendingRequestResult,
      // onFailure callback function
      onFailure: failureResult
  };
  sforce.connection.query("select Id, CreatedDate, ProcessInstance.TargetObjectId from "+
								"ProcessInstanceWorkitem where ActorId='" + options.getValue("actorId") + "' " +
								"and ProcessInstance.Status='Pending'", callback);
	entryList.displayMessage(strings.FETCH_APPROVALS);
}

/*
* Process Approval Requests here
*/
function pendingRequestResult(queryResults, source) {
	if (queryResults!=null && queryResults.size>0) {
		var records = queryResults.getArray("records");
		// loop through
		entryList.displayMessage(strings.INITIALIZE_VIEW);
		for(var i=0; i < records.length; i++) {
			debug.trace(records[i]);
			var targetId = records[i].getArray("ProcessInstance")[0].TargetObjectId;
			var modifiedDate = records[i].CreatedDate;
			debug.trace("TargetId:"+targetId);
			// cut first 3 symbols
			var keyPrefix = targetId.substring(0,3);
			// let's construct soql for targetId
			// send this query
			var entryData = {};
			// ID
			entryData['id'] = records[i].Id;
			// Publish Date
			entryData['published'] = sforce.internal.stringToDateTime(modifiedDate);
			debug.trace("get sobject by keyprefix:" + sObjectHashMap[keyPrefix]);
			// Link
			var detailLink = sObjectHashMap[keyPrefix].urldetail;
			detailLink = detailLink.replace("{ID}", targetId);
			entryData['link'] = detailLink;			
			debug.trace("obtain link url");
			debug.trace("object's keyPrefix:"+keyPrefix);
			debug.trace("object's name:"+sObjectHashMap[keyPrefix].name);
			// specify icon path
			entryData['icon'] = getIconSrc16(sObjectHashMap[keyPrefix].name);
			// describe layout
			var l = sforce.connection.describeLayout(sObjectHashMap[keyPrefix].name, null);
			var fields = getFieldListByLayouts(l.layouts.detailLayoutSections);
			var soql = "";
			if (fields=="") {
				soql = "select Id,Name from "+ sObjectHashMap[keyPrefix].name + " where Id = '" + targetId + "'"; 
			} else {
				soql = "select Id," + fields + " from "+ sObjectHashMap[keyPrefix].name + " where Id = '" + targetId + "'"; 
			}
			debug.trace("querying details");
			// run detail QR
			var detailQR = sforce.connection.query(soql);
			// Title
			entryData['title'] = detailQR.getArray("records")[0].Name;
			// generate content
			var contentHTML = detailHtmlContent.replace("{ID}", entryData.id);
			contentHTML = contentHTML.replace("{ID}", entryData.id);
			contentHTML = contentHTML.replace("{ID}", entryData.id);
			contentHTML = contentHTML.replace("{ID}", entryData.id);
			contentHTML = contentHTML.replace("{SFDC_URL}", entryData.link);
			contentHTML = contentHTML.replace("{SFDC_URL}", entryData.link);
			contentHTML = contentHTML.replace("{IMG_PATH}", getIconSrc32(sObjectHashMap[keyPrefix].name));
			// change title
			contentHTML = contentHTML.replace("{TITLE}", entryData.title);
			contentHTML = contentHTML.replace("{CONTENT}", generateDetailContent(detailQR.getArray("records")[0], l.layouts.detailLayoutSections));
			// content
			entryData['content'] = contentHTML;
			// Try to add this entry
			entryList.addItem(entryData);
			debug.trace("Item added");
		}
  } else {
		entryList.displayMessage(strings.NO_REQUESTS);
	}
}

function itemOnClick(entry) {
  //debug.trace('Entry item onclick event.');

  // Is a details view already open?
  if (currentEntry !== null) {
    var previousEntry = currentEntry;

    // We explicitly close so we know for sure that "currentEntry" is
    // null at this point.
    pluginHelper.closeDetailsView();

    // Same entry as before, no need to open it again.
    if (previousEntry == entry) {
      return;
    }
  }

  currentEntry = entry;

  var htmlDetailsView = new DetailsView();
  htmlDetailsView.html_content = true;

  htmlDetailsView.setContent('', undefined, entry.content, false, 0);

	// make communication between detail view and base view
	var externalObject = new Object();
	externalObject.approveEntity = approveEntity;
	externalObject.rejectEntity = rejectEntity;
	htmlDetailsView.external = externalObject;

  // Show the details view
  pluginHelper.showDetailsView(htmlDetailsView,
      entry.title,
      gddDetailsViewFlagToolbarOpen,
      function(flags) { detailsViewOnFeedback(flags, entry.link); }); 
}

// Handles when details view is closed.
// Here we want to open a link to the entry if the title bar is clicked.
function detailsViewOnFeedback(flags, url) {
  currentEntry = null;

  //debug.trace('detailsViewOnFeedback event.');

  // User clicked on the title bar link.
  if (flags == gddDetailsViewFlagToolbarOpen) {
    openEntryInBrowser(url);
  }
} 

function openEntryInBrowser(link) {
	var shell = new ActiveXObject('WScript.Shell');
	shell.run(link);
}

function approveEntity(id) {
	return processApprovalRequest(id, true);
}

function rejectEntity(id) {
	return processApprovalRequest(id, false);
}

function processApprovalRequest(id, isApprove) {
  try {
		var workitem = new sforce.ProcessWorkitemRequest();
		workitem.action = (isApprove==true) ? "Approve" : "Reject";
		workitem.workitemId = id;
		var workitems = [];
		workitems[workitems.length] = workitem;
		var res = sforce.connection.process(workitems);
		if (res[0].success=="false") {
			var error = res[0].getArray("errors")[0];
			if (error.statusCode=="INVALID_CROSS_REFERENCE_KEY") {
				return strings.INVALID_CROSS_REFERENCE_KEY;
			} else {
				return error.message;
			}
		} else {
			// need to delete item
			entryList.removeItem(id);
		}
	} catch (e) {
		return e.faultcode;
	}
	return "OK";
}